@@ -1,10 +1,9 @@
 """
 ldapobject.py - wraps class _ldap.LDAPObject
-written by Michael Stroeder <michael@stroeder.com>
 
 See http://python-ldap.sourceforge.net for details.
 
-\$Id: ldapobject.py,v 1.92 2005/11/03 09:09:43 stroeder Exp $
+\$Id: ldapobject.py,v 1.101 2008/04/19 18:31:33 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -40,19 +39,22 @@ from ldap.schema import SCHEMA_ATTRS
 from ldap.controls import LDAPControl,DecodeControlTuples,EncodeControlTuples
 from ldap import LDAPError
 
+
 class SimpleLDAPObject:
   """
   Drop-in wrapper class around _ldap.LDAPObject
   """
 
   CLASSATTR_OPTION_MAPPING = {
-    # taken from Modules/options.h
     "protocol_version":   ldap.OPT_PROTOCOL_VERSION,
     "deref":              ldap.OPT_DEREF,
     "referrals":          ldap.OPT_REFERRALS,
     "timelimit":          ldap.OPT_TIMELIMIT,
     "sizelimit":          ldap.OPT_SIZELIMIT,
     "network_timeout":    ldap.OPT_NETWORK_TIMEOUT,
+    "error_number":ldap.OPT_ERROR_NUMBER,
+    "error_string":ldap.OPT_ERROR_STRING,
+    "matched_dn":ldap.OPT_MATCHED_DN,
   }
 
   def __init__(
@@ -98,8 +100,8 @@ class SimpleLDAPObject:
       if __debug__ and self._trace_level>=2:
         self._trace_file.write('=> LDAPError - %s: %s\n' % (e.__class__.__name__,str(e)))
       raise
-    if __debug__ and self._trace_level>=2:
-      if result!=None and result!=(None,None):
+    if __debug__ and self._trace_level>=1:
+      if self._trace_level>=2 and result!=None and result!=(None,None):
         self._trace_file.write('=> result: %s\n' % (repr(result)))
     return result
 
@@ -134,6 +136,27 @@ class SimpleLDAPObject:
   def abandon(self,msgid):
     return self.abandon_ext(msgid,None,None)
 
+  def cancel(self,cancelid,serverctrls=None,clientctrls=None):
+    """
+    cancel(cancelid[,serverctrls=None[,clientctrls=None]]) -> int
+        Send cancels extended operation for an LDAP operation specified by cancelid.
+	The cancelid should be the message id of an outstanding LDAP operation as returned
+        by the asynchronous methods search(), modify() etc.  The caller
+        can expect that the result of an abandoned operation will not be
+        returned from a future call to result().
+	In opposite to abandon() this extended operation gets an result from
+	the server and thus should be preferred if the server supports it.
+    """
+    return self._ldap_call(self._l.cancel,cancelid,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+
+  def cancel_s(self,cancelid,serverctrls=None,clientctrls=None):
+    msgid = self.cancel(cancelid,serverctrls,clientctrls)
+    try:
+      res = self.result(msgid,all=1,timeout=self.timeout)
+    except (ldap.CANCELLED,ldap.SUCCESS):
+      res = None
+    return res
+
   def add_ext(self,dn,modlist,serverctrls=None,clientctrls=None):
     """
     add_ext(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> int
@@ -287,13 +310,13 @@ class SimpleLDAPObject:
         dn is the DN of the entry to modify, and modlist is the list
         of modifications to make to the entry.
 
-        Each element of the list modlist should be a tuple of the form
-        (mod_op,mod_type,mod_vals), where mod_op is the operation (one
-        of MOD_ADD, MOD_DELETE, or MOD_REPLACE), mod_type is a string
-        indicating the attribute type name, and mod_vals is either
-        a string value or a list of string values to add, delete or
-        replace respectively.  For the delete operation, mod_vals may
-        be None indicating that all attributes are to be deleted.
+	Each element of the list modlist should be a tuple of the form
+	(mod_op,mod_type,mod_vals), where mod_op is the operation (one of
+	MOD_ADD, MOD_DELETE, MOD_INCREMENT or MOD_REPLACE), mod_type is a
+	string indicating the attribute type name, and mod_vals is either a
+	string value or a list of string values to add, delete, increment by or
+	replace respectively.  For the delete operation, mod_vals may be None
+	indicating that all attributes are to be deleted.
 
         The asynchronous modify() returns the message id of the
         initiated request.
@@ -412,8 +435,12 @@ class SimpleLDAPObject:
   def result3(self,msgid=_ldap.RES_ANY,all=1,timeout=None):
     if timeout is None:
       timeout = self.timeout
-    rtype, rdata, rmsgid, serverctrls = self._ldap_call(self._l.result3,msgid,all,timeout)
-    decoded_serverctrls = DecodeControlTuples(serverctrls)
+    ldap_result = self._ldap_call(self._l.result3,msgid,all,timeout)
+    if ldap_result is None:
+      rtype, rdata, rmsgid, decoded_serverctrls = (None,None,None,None)
+    else:
+      rtype, rdata, rmsgid, serverctrls = ldap_result
+      decoded_serverctrls = DecodeControlTuples(serverctrls)
     return rtype, rdata, rmsgid, decoded_serverctrls
  
   def search_ext(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
@@ -668,12 +695,7 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     self._uri = uri
     self._options = {}
     self._last_bind = None
-    SimpleLDAPObject.__init__(
-      self,uri,
-      trace_level,
-      trace_file,
-      trace_stack_limit
-    )
+    SimpleLDAPObject.__init__(self,uri,trace_level,trace_file,trace_stack_limit)
     self._retry_max = retry_max
     self._retry_delay = retry_delay
     self._start_tls = 0
@@ -722,7 +744,9 @@ class ReconnectLDAPObject(SimpleLDAPObject):
           self.start_tls_s()
         # Repeat last simple or SASL bind
         self._apply_last_bind()
-      except:
+      except ldap.SERVER_DOWN,e:
+        SimpleLDAPObject.unbind_s(self)
+        del self._l
         if __debug__ and self._trace_level>=1:
           self._trace_file.write('*** %d. reconnect to %s failed\n' % (
             self._retry_max-reconnect_counter+1,uri
@@ -742,10 +766,14 @@ class ReconnectLDAPObject(SimpleLDAPObject):
         break
 
   def _apply_method_s(self,func,*args,**kwargs):
+    if not self.__dict__.has_key('_l'):
+       self.reconnect(self._uri)
     try:
       return func(self,*args,**kwargs)
     except ldap.SERVER_DOWN:
-      # Reconnect
+      SimpleLDAPObject.unbind_s(self)
+      del self._l
+      # Try to reconnect
       self.reconnect(self._uri)
       # Re-try last operation
       return func(self,*args,**kwargs)
@@ -773,6 +801,9 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   def add_ext_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.add_ext_s,*args,**kwargs)
 
+  def cancel_s(self,*args,**kwargs):
+    return self._apply_method_s(SimpleLDAPObject.cancel_s,*args,**kwargs)
+
   def compare_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.compare_s,*args,**kwargs)
 
@@ -788,6 +819,9 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   def search_ext_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.search_ext_s,*args,**kwargs)
 
+  def whoami_s(self,*args,**kwargs):
+    return self._apply_method_s(SimpleLDAPObject.whoami_s,*args,**kwargs)
+
 
 class SmartLDAPObject(ReconnectLDAPObject):
   """
@@ -851,7 +885,7 @@ class SmartLDAPObject(ReconnectLDAPObject):
     except ldap.PROTOCOL_ERROR:
       # Drop connection completely
       self.unbind_s() ; del self._l
-      self._l = self._ldap_call(_ldap.initialize,self._uri)
+      self._l = ldap.functions._ldap_function_call(_ldap.initialize,self._uri)
       self.protocol_version = ldap.VERSION2
       self.simple_bind_s(who,cred)
     # Try to start TLS if requested
