@@ -3,7 +3,7 @@ schema.py - support for subSchemaSubEntry information
 
 See http://python-ldap.sourceforge.net for details.
 
-\$Id: models.py,v 1.33 2008/03/26 16:42:22 stroeder Exp $
+\$Id: models.py,v 1.36 2008/06/23 23:06:16 stroeder Exp $
 """
 
 import UserDict,ldap.cidict
@@ -495,7 +495,7 @@ class NameForm(SchemaElement):
     'NAME':(()),
     'DESC':(None,),
     'OBSOLETE':None,
-    'OC':(()),
+    'OC':(None,),
     'MUST':(()),
     'MAY':(()),
   }
@@ -504,13 +504,13 @@ class NameForm(SchemaElement):
     self.names = d['NAME']
     self.desc = d['DESC'][0]
     self.obsolete = d['OBSOLETE']!=None
-    self.oc = d['OC']
+    self.oc = d['OC'][0]
     self.must = d['MUST']
     self.may = d['MAY']
     assert type(self.names)==TupleType
     assert self.desc is None or type(self.desc)==StringType
     assert type(self.obsolete)==BooleanType and (self.obsolete==0 or self.obsolete==1)
-    assert type(self.oc)==TupleType
+    assert type(self.oc)==StringType
     assert type(self.must)==TupleType
     assert type(self.may)==TupleType
     return
@@ -520,7 +520,7 @@ class NameForm(SchemaElement):
     result.append(self.key_list('NAME',self.names,quoted=1))
     result.append(self.key_attr('DESC',self.desc,quoted=1))
     result.append({0:'',1:' OBSOLETE'}[self.obsolete])
-    result.append(self.key_list('OC',self.oc,sep=' $ '))
+    result.append(self.key_attr('OC',self.oc))
     result.append(self.key_list('MUST',self.must,sep=' $ '))
     result.append(self.key_list('MAY',self.may,sep=' $ '))
     return '( %s )' % ''.join(result)
@@ -564,7 +564,7 @@ class Entry(UserDict.UserDict):
       self[key] = dict[key]
 
   def __contains__(self,key):
-    return self.has_key(self,key)
+    return self.has_key(key)
 
   def __getitem__(self,nameoroid):
     return self.data[self._at2key(nameoroid)]
