@@ -1,14 +1,13 @@
 """
 ldap.schema.tokenizer - Low-level parsing functions for schema element strings
-written by Michael Stroeder <michael@stroeder.com>
 
 See http://python-ldap.sourceforge.net for details.
 
-\$Id: tokenizer.py,v 1.8 2003/11/22 20:15:44 stroeder Exp $
+\$Id: tokenizer.py,v 1.11 2008/03/10 08:34:30 stroeder Exp $
 """
 
 
-def split_tokens(s):
+def split_tokens(s,keywordDict):
   """
   Returns list of syntax elements with quotes and spaces
   stripped.
