@@ -1,9 +1,6 @@
-/* David Leonard <david.leonard@csee.uq.edu.au>, 1999. Public domain. */
-
-/* 
- * constants defined for LDAP
- * $Id: constants.c,v 1.30 2007/07/17 19:37:27 stroeder Exp $
- */
+/* constants defined for LDAP
+ * See http://python-ldap.sourceforge.net for details.
+ * $Id: constants.c,v 1.34 2008/03/26 16:17:26 stroeder Exp $ */
 
 #include "common.h"
 #include "constants.h"
@@ -152,9 +149,12 @@ LDAPinit_constants( PyObject* d )
 	add_int(d,OPT_CLIENT_CONTROLS);
 	add_int(d,OPT_API_FEATURE_INFO);
 	add_int(d,OPT_HOST_NAME);
+
+	/* For backward-compability with OpenLDAP 2.3 libs this is defined in ldap/__init__.py */
+	/* add_int(d,OPT_DIAGNOSTIC_MESSAGE); */
+
 	add_int(d,OPT_ERROR_STRING);
 	add_int(d,OPT_MATCHED_DN);
-	add_int(d,OPT_PRIVATE_EXTENSION_BASE);
 	add_int(d,OPT_DEBUG_LEVEL);
 	add_int(d,OPT_TIMEOUT);
 	add_int(d,OPT_REFHOPLIMIT);
